(******************************************************************************)
(* plx parser (l00)                                                           *)
(* (C) 2010..2021 Steffen Solyga <solyga@gmx.de>                              *)
(******************************************************************************)
IMPLEMENTATION MODULE Prsr;

IMPORT FIO;
FROM Scnr IMPORT Read, TSymbol, sym, num, pos;
FROM Comp IMPORT TValue, TOperation, Val, Opr;

CONST
  err ::= FIO.ErrorOutput;


PROCEDURE Err( s: ARRAY OF CHAR );
BEGIN
  FIO.WrStr( err, 'Prsr (' );
  FIO.WrCard( err, pos.l, 0 );
  FIO.WrChar( err, ',' );
  FIO.WrCard( err, pos.c, 0 );
  FIO.WrStr( err, '): ' );
  FIO.WrStr( err, s );
  FIO.WrStr( err, '.' );
  FIO.WrLn( err );
  HALT;
END Err;

PROCEDURE Expression();

  PROCEDURE Factor();
  BEGIN
    IF sym = sNum THEN
      IF num > MAX( TValue ) THEN Err( 'Number too large' ) END;
      Val( TValue(num) ); Read();
    ELSIF sym = sLPar THEN
      Read(); Expression();
      IF sym = sRPar THEN Read() ELSE Err( 'Expected operator or ")"' ) END;
    ELSE
      Err( 'Expected number or "("' );
    END;
  END Factor;

  PROCEDURE Term();
  VAR op: TSymbol;
  BEGIN
    Factor();
    WHILE ( sym = sStar ) OR ( sym = sSlash ) DO
      op := sym; Read(); Factor();
      IF op = sStar THEN Opr( oMul ) ELSE Opr( oDiv ) END;
    END;
  END Term;

VAR op: TSymbol;
BEGIN
  IF ( sym = sPlus ) OR ( sym = sMinus ) THEN
    op := sym; Read(); Term();
    IF op = sMinus THEN Opr( oNeg ) END;
  ELSE
    Term();
  END;
  WHILE ( sym = sPlus ) OR ( sym = sMinus ) DO
    op := sym; Read(); Term();
    IF op = sPlus THEN Opr( oAdd ) ELSE Opr( oSub ) END;
  END;
END Expression;


PROCEDURE Run();
BEGIN
  Read(); Expression(); Opr( oWrt );
  WHILE sym = sSemi DO Read(); Expression(); Opr( oWrt ) END;
  IF sym # sDot THEN Err( 'Expected operator, ";" or "."' ) END;
END Run;

BEGIN
END Prsr.
